%% Hopper simulation w/ max extension
%%
%% This is the main simulation script for the hopper
%% 	2-body vertical simulation.  It uses the ode45
%% 	integration function with event checking.
%% 
%% It uses two mode, or state, variables: state_foot and
%% 	state_p to keep track of the system's 4 state.  
%% 
%% State_foot has two modes:
%% 	GROUND				:	Foot is attached to the ground and
%% 						  		the body is attached by the spring and damper
%% 	AIR					:	Both masses are airborne and connected
%% 						  		by the spring and damper
%% State_act has two modes:
%% 	ON		:	Force is turned on
%% 	OFF     :	Force is turned off
%%
%% State_ext has two modes:
%%  FREE    :   Spring is not hitting its max extension
%%  LIMIT   :   Spring is at max extension
%% 
%% Basically, it calls hopper_ode, sending it the two mode
%% 	variables and letting it figure out which eom's to use
%% When the integration stops, it checks to see what event triggered it
%%		then restarts the integration with a new set of initial conditions
%%		as appropriate.
%%
%% Jorge Cham - 112600 - Basic Functionality
%% Jonathan Karpick - modified March 2006 - simplified piston to a simple
%%      force actuator

%clear all;

%load_constants;		% this loads into the current workspace all the
					% global constants
                     

%function [tout,yout,stout_foot,stout_act,ss_height,periodicity]=hopper_sim(hopper,t_start,t_final,y0)
function [data,ss_height,periodicity]=hopper_sim(hopper,t_start,t_final,y0)

options = odeset('Events',@hopper_events,'RelTol',1e-6);
							% turn on the event checker
	
tout = t_start;		% Initialize the output vectors
yout = y0.';
teout = [];
yeout = [];
ieout = [];

hopper.state_foot = hopper.GROUND;		% Set initial mode variables
hopper.state_act = hopper.OFF;
if yout(1) <= hopper.max_ext            % Max extension setting is a bit tricky
    hopper.state_ext = hopper.FREE;
else
    hopper.state_ext = hopper.LIMIT;
    yout(1) = hopper.max_ext
    y0(1) = hopper.max_ext
    if yout(2) > 0
        yout(2) = 0;
        y0(2) = 0;
    end
end

stout_foot = hopper.state_foot;
stout_act = hopper.state_act;
stout_ext = hopper.state_ext;

i=1;
go = 1;
ss_period_count=0;
t_alert_int=5;
t_alert=t_alert_int;

while (go)
  % Solve until the first terminal event.
  [t,y,te,ye,ie] = ode45(@hopper_ode,[t_start t_final],y0,options,hopper);
  
  % Accumulate output vectors.
  nt = length(t);
  tout = [tout; t(2:nt)];
  yout = [yout; y(2:nt,:)];
  stout_foot = [stout_foot; hopper.state_foot*ones(nt-1,1)];
  stout_act = [stout_act; hopper.state_act*ones(nt-1,1)];
  stout_ext = [stout_ext; hopper.state_ext*ones(nt-1,1)];
  
  teout = [teout; te];          % Events at tstart are never reported. 
  yeout = [yeout; ye];
  ieout = [ieout; ie];

  % Set the new initial conditions to the last state (for now).
  y0(1) = y(length(t),1);
  y0(2) = y(length(t),2);
  y0(3) = y(length(t),3);
  y0(4) = y(length(t),4);
  
  
  % EVENTS
  if ~isempty(ie)
      if hopper.state_act == hopper.ON
          Fappl = hopper.F;
      else
          Fappl = 0;
      end

      switch hopper.state_foot
         case hopper.AIR
             switch ie(length(ie))
                case hopper.TOUCHDOWN
                     hopper.state_foot = hopper.GROUND;
                     y0(3) = 0;	% put foot on ground, body keeps moving with same velocity
                     y0(4) = 0;
                     if y0(2) < 0
                         hopper.state_ext = hopper.FREE;
                     end
                case hopper.BEGIN_FORCE
                     hopper.state_act = hopper.ON;
                     [go,ss_height,periodicity]=check_ss_period(tout,yout,hopper);
                case hopper.END_FORCE
                     hopper.state_act = hopper.OFF;
                     if hopper.state_ext == hopper.LIMIT
                         Fes = -hopper.m1*hopper.g-hopper.k*(y0(1)-y0(3))-hopper.b*(y0(2)-y0(4));
                         if Fes<=0
                             hopper.state_ext = hopper.FREE;
                         end
                     end
                 case hopper.HIT_LIMIT
                     hopper.state_ext = ~hopper.state_ext;  % assumes only 0 and 1
                     if hopper.state_ext == hopper.LIMIT
                         y0(1) = hopper.max_ext;
                         y0(2) = (hopper.m1*y0(2)+hopper.m2*y0(4))/(hopper.m1+hopper.m2);
                         y0(4) = y0(2);  %conservation of momentum
                         Fes = Fappl-hopper.m1*hopper.g-hopper.k*(y0(1)-y0(3))-hopper.b*(y0(2)-y0(4));
                         if Fes<=0
                             hopper.state_ext = hopper.FREE;
                         end
                     end
             end
          case hopper.GROUND
             switch ie(length(ie))
                 case hopper.TAKEOFF
                     hopper.state_foot = hopper.AIR;	%all states remain the same
                 case hopper.BEGIN_FORCE
                     hopper.state_act = hopper.ON;
                     [go,ss_height,periodicity]=check_ss_period(tout,yout,hopper);
                 case hopper.END_FORCE
                     hopper.state_act = hopper.OFF;
                     if hopper.state_ext == hopper.LIMIT
                         Fes = -hopper.m1*hopper.g-hopper.k*(y0(1)-y0(3))-hopper.b*(y0(2)-y0(4));
                         if Fes<=0
                             hopper.state_ext = hopper.FREE;
                         end
                     end
                     if hopper.state_ext == hopper.FREE
                         Fes = 0;
                     else
                         Fes = -hopper.m1*hopper.g-hopper.k*(y0(1)-y0(3))-hopper.b*(y0(2)-y0(4));
                     end
                     Fn=hopper.m2*hopper.g-hopper.k*y0(1)-hopper.b*y0(2)-Fes;
                     if Fn<=0
                         hopper.state_foot=hopper.AIR;
                     end
                 case hopper.HIT_LIMIT
                     hopper.state_ext = ~hopper.state_ext;  % assumes only 0 and 1
                     if hopper.state_ext == hopper.LIMIT
                         y0(1) = hopper.max_ext;
                         y0(2) = (hopper.m1*y0(2)+hopper.m2*y0(4))/(hopper.m1+hopper.m2);
                         y0(4) = y0(2);  %conservation of momentum
                         Fes = Fappl-hopper.m1*hopper.g-hopper.k*(y0(1)-y0(3))-hopper.b*(y0(2)-y0(4));
                         if Fes<=0
                             hopper.state_ext = hopper.FREE;
                         end
                         if y0(4) > 0  %takeoff
                             hopper.state_foot = hopper.AIR;
                         end
                     end
             end  % switch of ie
      end  % switch of hopper.state_foot
  end % if(~isempty(ie))
  t_start = t(nt);
  if t_start>=t_final
      t_final=t_final*2;
  end
  i=i+1;
  if t_final > 100  %Time out at 100seconds
      go=0;
  end
  if t_alert < t(nt)
      sprintf('t=%d',t_alert)
      t_alert=t_alert+t_alert_int;
  end
end  %while

data.tout = tout;
data.yout = yout;
data.stout_foot = stout_foot;
data.stout_act = stout_act;
data.stout_ext = stout_ext;
data.hopper = hopper;
data.ss_height = ss_height;
data.periodicity = periodicity;

%hopper_plot;	

end  %function
